/* ICV150.h */

#ifndef _ICV150_h
#define _ICV150_h


/*----------------------------------------------------------------------
 * Definitions
 *---------------------------------------------------------------------*/

#define		u32	unsigned long
#define		u16	unsigned short
#define		u8	unsigned char


/*----------------------------------------------------------------------
 * Constants
 *---------------------------------------------------------------------*/

/*ICV150 boards properties*/
#define		MAX_BOARDS	4		/*maximum number of ICV150 boards*/
#define		MAX_GAIN	10		/*maximum gain value*/
#define		MAX_CHANNELS	256		/*max number of channels per board*/
#define		DEF_CHANNELS	32		/*default number of channels per board*/
#define		ICV150_SIZE	4096		/*bytes occupation of one
						 *ICV150 board*/

/*ICV150 default config*/
#define		IO_BOARD_ADR	0xE17C30	/*default ICV150 address*/

/*ioctl() cmd constants*/
#define		RD_CHANNEL	0x10
#define		RD_ALL_CHANNELS	0x11
#define		RD_GAIN		0x12
#define		RD_ALL_GAINS	0x13
#define		SET_NUMBER	0x14
#define		SET_GAIN	0x15
#define		SET_ALL_GAINS	0x16
#define		START		0x17
#define		EXT_TRIG	0x18
#define		STOP		0x19
#define		STORE		0x1A
#define		RECALL		0x1B
#define		DEBUG		0x1C


/*----------------------------------------------------------------------
 * Initialization parameters
 *---------------------------------------------------------------------*/

static u32 	*io_adr[MAX_BOARDS]	= {	(u32*) IO_BOARD_ADR,
						NULL,
						NULL,
						NULL	 };


/*----------------------------------------------------------------------
 * Conversion tables for the ADAS ICV150 irq level register
 *---------------------------------------------------------------------*/

/*
 * As the interrupt level of the ICV150 board is written in negative logic,
 * we use this array to convert the values.

static u8	levelconv[8] = {0, 0xFD, 0xFB, 0xF7, 0xEF, 0xDF, 0xB7, 0x7F};
 */


/*----------------------------------------------------------------------
 * Structures that map the ADAS ICV150 I/O registers
 *---------------------------------------------------------------------*/

struct icv150regs
{
	u16	Data[MAX_CHANNELS]; /*Data acquisition result's of the channels
				 *on RD. Gain's codes on WR*/
	u16	Start;		/*Start continuous automatic scanning (dummy RD
				 *or WR)*/
	u8	pad1[254];	/*not used*/
	u16	Stop_Status;	/*Stop automatic scanning on WR. Contains board
				 *status in LSB on RD*/
	u8	pad2[254];	/*not used*/
	u16	Trig;		/*Start one automatic scanning (dummy RD or WR)*/
	u8	pad3[254];	/*not used*/
	u16	Ext;		/*Enable the board to trig on external clock's
				 *rising edges (dummy RD or WR)*/
	u8	pad4[254];	/*not used*/
	u16	Array;		/*Recall the gain's codes from EEPROM*/
	u8	pad5[254];	/*not used*/
	u16	Store;		/*Store the gain's codes in EEPROM*/
	u8	pad6[254];	/*not used*/
	u16	RWit;		/*Acks the end of conversion on RD. Write
				 *interrupt vector, level and IEN on WR*/
	u8	pad7[1022];	/*not used*/
	u16	Channels;	/*Program the number of channels to be scanned
				 *(WR only)*/
	u8	pad8[510];	/*not used*/
	u16	Gain[MAX_CHANNELS]; /*Gain codes of the channels in the MSByte*/
};

typedef struct icv150regs ICV150_REGS;


/*----------------------------------------------------------------------
 * Structure that keep the ADAS ICV150 board state (one per board)
 *---------------------------------------------------------------------*/

struct icv150state
{
	u8	Channels;		/*Number of the last channel to scan
					 *(scan is from channel 0 to 'Channels')*/
	u8	External;		/*1->external trigger enabled*/
};

typedef struct icv150state ICV150_STATE;


/*----------------------------------------------------------------------
 * Functions prototypes 
 *---------------------------------------------------------------------*/
/*
static int ICV150_init();
static int ICV150_ioctl (struct inode*, struct file*, int, void*);
static int ICV150_open (struct inode*, struct file*);
static void ICV150_release (struct inode*, struct file*);
int init_module ();
void cleanup_module (); 
*/


#endif	/* _ICV150_h */
