#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/file.h>
#include <sys/time.h>
#include <icv150.h>



#define N_RD_ALL 10 
#define N_RD  4
#define N_CH  32
#define DBG_LVL  4

extern int errno;
int 	FD;
char 	dev_name[80];
u8 	action;
long 	n_rd = -1;  /* default: use N_RD and N_RD_ALL */
int 	dbg_lvl = DBG_LVL; /* default DEBUG level */
unsigned long cnt=1;
unsigned long nb_err=0;

void my_signal_handler(int );



void main (unsigned int argc, char **argv)
{
 int 	i, j;
 u8 	cmd;
 u16 	arg[MAX_CHANNELS];

 struct timeval before, after;
/* struct timespec millisec={0, 100000};*/
 float elapsed;

 struct sigaction sa;




 if (argc < 3) 
 {
  printf("\nUsage: %s device read_type [dbg [n_rd]]\n", argv[0]);
  printf("\tdevice   = /dev/icv150_1 | /dev/icv105_2 ...\n");
  printf("\tread_type= 1=RD_ALL_CHANNELS | 0=RD_CHANNELS\n");
  printf("\tdbg      = driveur debug level, 0=min, 4=max\n");
  printf("\tn_rd     = number of calls\n");
  printf("\n");
  exit(-1);
 }

 if (argc > 1) 
 {
  strcpy(dev_name, argv[1]);
 } else {
  strcpy(dev_name, "/dev/icv150_1");
 }

 FD = open (dev_name, O_RDWR, 0x666);
 printf("\nopened %s File Descriptor = %d\n", dev_name, FD);

 if (FD == -1)
 {
  printf("\n\nError, unable to open the ICV150 board (errno=%d)\n",errno);
  exit (0);
 }


 if (argc > 3)
 {
  if(sscanf(argv[3],"%d",&dbg_lvl) != 1)
   dbg_lvl = DBG_LVL;
 }

 if (argc > 4)
 {
  if(sscanf(argv[4],"%ld",&n_rd) != 1)
   n_rd = -1;
 }

 if ((argc > 2) && !strcmp(argv[2],"1"))
 {
  action = RD_ALL_CHANNELS;

  if(n_rd == -1)
   n_rd = N_RD_ALL;
  
  printf("doing %ld x RD_ALL_CHANNELS\n", n_rd);
 } else {
  action = RD_CHANNEL;

  if(n_rd == -1)
   n_rd = N_RD;
  
  printf("doing %ld x %d x RD_CHANNEL\n", n_rd, N_CH);
 }


 cmd = DEBUG;

 printf("Setting DEBUG level : %d\n", dbg_lvl);
 arg[0] = (u8) (dbg_lvl & 0x00FF);
 if ( ioctl(FD, cmd, arg) != 0)
 {
  printf("Error setting DEBUG level: errno = %d\n", errno);
  exit(-1);
  close(FD);
 }

 printf("\n");




 sa.sa_handler = my_signal_handler;
 sa.sa_flags   = 0;
 sigaction(SIGINT, &sa, NULL);
 sigaction(SIGQUIT, &sa, NULL);


 for (;;)
 {
  switch (action) 
  {
   case (RD_ALL_CHANNELS) :

    gettimeofday(&before, NULL);
    for (j=0; j<n_rd; j++)
    {
     cmd = RD_ALL_CHANNELS;

     if ( ioctl(FD, cmd, arg) != 0)
     {
      printf("Error on RD_ALL_CHANNELS : (errno = %d)\n", errno);
      nb_err++;
     }

     /*nanosleep(&millisec,NULL);*/
    }
    gettimeofday(&after, NULL);
    elapsed = after.tv_sec - before.tv_sec;
    elapsed = elapsed*1000. + (float)(after.tv_usec-before.tv_usec)/1000.;
    printf("%12ld %3ld: elapsed %6.3f ms for %ld reads = %6.3f ms per ioctl()\n",
          cnt++, nb_err, elapsed, n_rd, elapsed/(float)n_rd);

    break;

   case (RD_CHANNEL) :

    gettimeofday(&before, NULL);
    for (j=0; j<n_rd; j++)
    {
     cmd = RD_CHANNEL;

     for (i=0; i<N_CH; i++)
     {
      arg[0] = (u8) (i & 0x00FF);

      if ( ioctl(FD, cmd, arg) != 0)
      {
       printf("Error on RD_CHANNELS : (errno = %d)\n", errno);
       nb_err++;
      }
     }
    }
    gettimeofday(&after, NULL);
    elapsed = after.tv_sec - before.tv_sec;
    elapsed = elapsed*1000. + (float)(after.tv_usec-before.tv_usec)/1000.;
    printf("%12ld %3ld: elapsed %6.3f ms for %ld reads = %6.3f ms per ioctl()\n",
          cnt++, nb_err, elapsed, n_rd*N_CH, elapsed/(float)(n_rd*N_CH));

    break;
  }
 }

} 



void my_signal_handler(int s)
{
 printf("\nclosing %s File Descriptor = %d\n", dev_name, FD);
 close(FD);

 if(action == RD_ALL_CHANNELS)
  printf("done %ld x RD_ALL_CHANNELS\n", n_rd);
 else
  printf("done %ld x %d x RD_CHANNEL\n", n_rd, N_CH);

 printf("Set DEBUG level : %d\n", dbg_lvl);
 printf("Number of errors on ioctl() : %ld\n",nb_err);
 printf("\n");

 exit(0);
}
